  //=============================================================================
// File: OAuthSignatureUtil.java
// Date        Author     Changes
// ----------------------------------------------------------------------------
// 07/15/2013  knagarajan@modulant.com   Created
//=============================================================================
package com.logistic.webservice;

import java.io.UnsupportedEncodingException;

import java.net.URLEncoder;

import java.security.GeneralSecurityException;

import java.util.LinkedHashMap;
import java.util.Map;
import java.util.TreeMap;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;

//Apache Commons Libraries used for the Nonce & Base64
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang3.RandomStringUtils;

/**
 *
 * @author Knagarajan - Modulant
 *
 * @since 15 July 2013
 *
 */
public class OAuthSignatureUtil {

    //OAuth Parameter Names
    public static final String PARAM_OAUTH_CONSUMER_KEY = "oauth_consumer_key";
    public static final String PARAM_OAUTH_SIGNATURE = "oauth_signature";
    public static final String PARAM_OAUTH_APP_ID = "oauth_application_id";
    public static final String PARAM_OAUTH_SIGNATURE_METHOD = "oauth_signature_method";
    public static final String PARAM_OAUTH_TIMSTAMP = "oauth_timestamp";
    public static final String PARAM_OAUTH_NONCE = "oauth_nonce";
    public static final String PARAM_AUTH_NAME = "x-authorization";

    //Common Keys used both consumer & provider
    public static final String MOBILE_APP_ID = "MYECOAPP";
    public static final String CONSUMER_KEY = "MYECOMOBILEUSER";
    public static final String SECRET_KEY = "ECO$99_ws!";
    public static final String SIGNATURE_METHOD = "HmacSHA1";

    public static String generateConsumerSignature()
            throws GeneralSecurityException, UnsupportedEncodingException {
        String xauth = null;

        // Create the OAuth parameter name/value pair
        Map<String, String> oauthParams = new LinkedHashMap<String, String>();

        oauthParams.put(PARAM_OAUTH_CONSUMER_KEY, CONSUMER_KEY);
        oauthParams.put(PARAM_OAUTH_APP_ID, MOBILE_APP_ID);
        oauthParams.put(PARAM_OAUTH_SIGNATURE_METHOD, SIGNATURE_METHOD);
        oauthParams.put(
                PARAM_OAUTH_TIMSTAMP,
                getTimestamp());
        oauthParams.put(
                PARAM_OAUTH_NONCE,
                getNonce());

        System.out.println("oauthParams = " + oauthParams);

        // Get the OAuth 1.0 Signature
        String signature = generateSignature(oauthParams, SECRET_KEY);

        System.out.println(String.format("OAuth= %s", signature));

        // Add the oauth_signature parameter to the set of OAuth Parameters
        oauthParams.put(PARAM_OAUTH_SIGNATURE, signature);

        // Genterate a string of comma delimited: keyName="URL-encoded(value)" pairs
        int i = 0;
        StringBuilder sb = new StringBuilder();
        Object[] keyNames = oauthParams.keySet().toArray();

        for (Object keyName : keyNames) {
            //No Need to send all the parameters as plain text over the URl as parameter. 
            // Just send timestamp, nonce & the signature
            if (PARAM_OAUTH_TIMSTAMP.equals(keyName)
                    || PARAM_OAUTH_NONCE.equals(keyName)
                    || PARAM_OAUTH_SIGNATURE.equals(keyName)) {
                String value = oauthParams.get((String) keyName);

                sb.append(keyName).append("=\"").append(
                        URLEncoder.encode(value, "UTF-8")).append("\"");
                i++;

                if (3 > i /*keyNames.length > i*/) {
                    sb.append(",");
                }
            }
        }

        // Build the X-Authorization request header
        xauth = String.format(
                PARAM_AUTH_NAME + "=%s",
                sb.toString());

        return xauth;
    }

    /**
     * Generates a random nonce
     *
     * @return A unique identifier for the request
     */
    private static String getNonce() {
        return RandomStringUtils.randomAlphanumeric(32);
    }

    /**
     * Generates an integer representing the number of seconds since the unix
     * epoch using the date/time the request is issued
     *
     * @return A timestamp for the request
     */
    private static String getTimestamp() {
        return Long.toString((System.currentTimeMillis() / 1000));
    }

    /**
     * Generates an OAuth 1.0 signature
     *
     * @param oauthParams The associative set of signable oAuth parameters
     * @param secret Alphanumeric string used to validate the identity of the
     * education partner (Private Key)
     *
     * @return A string containing the Base64-encoded signature digest
     *
     * @throws UnsupportedEncodingException
     */
    public static String generateSignature(Map<String, String> oauthParams,
            String secret)
            throws UnsupportedEncodingException, GeneralSecurityException {
        // Construct the URL-encoded OAuth parameter portion of the signature base string
        String encodedParams = normalizeParams(oauthParams);

        //System.out.println("encodedParams = " + encodedParams);
        // Build the signature base string to be signed with the Consumer Secret
        String baseString = String.format("%s", encodedParams);

        //System.out.println("baseString = " + baseString);
        return generateHmacSHA1Signature(baseString, secret);
    }

    /**
     * Normalizes all OAuth signable parameters and url query parameters
     * according to OAuth 1.0
     *
     * @param oauthParams The associative set of signable oAuth parameters
     *
     * @return A string containing normalized and encoded oAuth parameters
     *
     * @throws UnsupportedEncodingException
     */
    public static String normalizeParams(Map<String, String> oauthParams)
            throws UnsupportedEncodingException {
        // Use a new LinkedHashMap for the OAuth signature creation 
        Map<String, String> kvpParams = new LinkedHashMap<String, String>();

        kvpParams.putAll(oauthParams);

        // Sort the parameters in lexicographical order, 1st by Key then by Value; separate with ("=")
        TreeMap<String, String> sortedParams = new TreeMap<String, String>(String.CASE_INSENSITIVE_ORDER);

        sortedParams.putAll(kvpParams);
          //System.out.println("normalizeParams - sortedParams " + sortedParams);

        // Remove unwantted characters and replace the comma delimiter with an ampersand
        String stringParams = sortedParams.toString().replaceAll("[{} ]", "");

        stringParams = stringParams.replace(",", "&");
        System.out.println("\n\nnormalizeParams - stringParams " + stringParams);

        // URL-encode the equals ("%3D") and ampersand ("%26")
        String encodedParams = URLEncoder.encode(stringParams, "UTF-8");

        //System.out.println("\n\nnormalizeParams - encodedParams " + encodedParams);
        return encodedParams;
    }

    public static String generateHmacSHA1Signature(String data, String key)
            throws GeneralSecurityException {
        System.out.println("\n\nBefore HmacSHA1 : " + data);
        byte[] hmacData = null;

        try {
            SecretKeySpec secretKey = new SecretKeySpec(key.getBytes("UTF-8"),
                    SIGNATURE_METHOD);

            Mac mac = Mac.getInstance(SIGNATURE_METHOD);

            mac.init(secretKey);

            hmacData = mac.doFinal(data.getBytes("UTF-8"));

            //System.out.println("\n\nAfter HmacSHA1 : "+hmacData);
            StringBuilder sb = new StringBuilder();
            for (byte b : hmacData) {
                sb.append(String.format("%02X", b));
            }
            String hash = sb.toString();

            //String s = new String(hmacData);              
            System.out.println("After HmacSHA1 : " + hash);

            String base64Encoded = new Base64().encodeAsString(hmacData);
            System.out.println("\n\nBase64 Encoded : " + base64Encoded);

            //System.out.println("\n\nBase64 Decoded : "+new Base64().decodeBase64(base64Encoded));
            return new Base64().encodeAsString(hmacData);
        } catch (UnsupportedEncodingException e) {
            // TODO: handle exception
            throw new GeneralSecurityException(e);
        }
    }
}
